unit Unit1;

interface

uses
  System.SysUtils, System.Classes, JS, Web, WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, WEBLib.Grids, Vcl.Controls, Vcl.Grids,
  WEBLib.DBCtrls, Data.DB, WEBLib.DB, WEBLib.IndexedDb, Vcl.StdCtrls,
  WEBLib.StdCtrls, Vcl.Menus, WEBLib.Menus, WEBLib.ExtCtrls;

type
  TForm1 = class(TWebForm)
    WebIndexedDbClientDataset1: TWebIndexedDbClientDataset;
    btnClrAndRefillTable: TWebButton;
    WebLabel1: TWebLabel;
    btnCountRecords: TWebButton;
    btnUpdateTable: TWebButton;
    WebEdit1: TWebEdit;
    LogMemo: TWebMemo;
    btnGreen: TWebButton;
    btnRose: TWebButton;
    btnClrFilter: TWebButton;
    WebDBGrid1: TWebDBGrid;
    WebDataSource1: TWebDataSource;
    btnGray: TWebButton;
    btnGoldenRod: TWebButton;
    pnlMenu: TWebPanel;
    WebMainMenu1: TWebMainMenu;
    Listing: TMenuItem;
    Filter: TMenuItem;
    fltrGenre: TMenuItem;
    fltrTitle: TMenuItem;
    fltrNone: TMenuItem;
    Scheduled: TMenuItem;
    History: TMenuItem;
    Options: TMenuItem;
    RefreshEPG: TMenuItem;
    RefreshHistory1: TMenuItem;
    ChangeHTPC1: TMenuItem;
    ViewLog1: TMenuItem;
    Settings1: TMenuItem;
    btnMenuTest: TWebButton;
    btnLocateRecord: TWebButton;
    WebSpinEdit1: TWebSpinEdit;
    procedure btnClrFilterClick(Sender: TObject);
    procedure btnRoseClick(Sender: TObject);
    procedure btnGreenClick(Sender: TObject);
    procedure btnGrayClick(Sender: TObject);
    procedure btnGoldenRodClick(Sender: TObject);
    [async]
    procedure WebFormCreate(Sender: TObject);
    [async] procedure btnClrAndRefillTableClick(Sender: TObject);
    [async] procedure btnCountRecordsClick(Sender: TObject);
    [async] procedure btnUpdateTableClick(Sender: TObject);
    [async] procedure CountWIDBCDSRecords;
    procedure WebDBGrid1GetCellClass(Sender: TObject; ACol, ARow: Integer;
      AField: TField; AValue: string; var AClassName: string);
    procedure btnMenuTestClick(Sender: TObject);
    procedure btnLocateRecordClick(Sender: TObject);

  private
    { Private declarations }
    [async] procedure SetupWIDBCDS;
    procedure FormRecord(j: Integer);
    [async] procedure SetFilter(ClassName: string);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses StrUtils, DateUtils;

{$R *.dfm}

procedure Log(Text: string);
var NewLine: string;
begin
  NewLine := TimeToStr(Now) + '---' + Text;
  Form1.LogMemo.Lines.Add(NewLine);
  console.log(NewLine);
end;

procedure TForm1.FormRecord(j: Integer);
var
  i: Integer;
  AColor: string;
begin
  WebIndexedDBClientDataSet1.Fields[0].Value := j;
  for i := 1 to WebIndexedDbClientDataset1.FieldCount-2 do
    if WebIndexedDBClientDataSet1.Fields[i].DataType = ftString then
      WebIndexedDBClientDataSet1.Fields[i].Value := 'wild_guess' + j.ToString
    else
      WebIndexedDBClientDataSet1.Fields[i].Value := Now;
  case j mod 4 of
    0: AColor := 'rose'; // 'goldenRod';
    1: AColor := 'green';
    2: AColor := 'gray';
    3: AColor := 'goldenRod'; // 'rose';
  end;
  WebIndexedDBClientDataSet1.FieldByName('Class').Value := AColor;
end;


procedure TForm1.btnClrAndRefillTableClick(Sender: TObject);
var
  j: Integer;
  Start: TTime;
begin
  Log('Starting LoadWIDBCDS, DB is ' + IfThen(not WebIndexedDBClientDataSet1.Active, 'not ') + 'Active');
  Start := Now;
  btnClrAndRefillTable.Caption := 'Working...';
  {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
  if WebIndexedDBClientDataSet1.Filtered then WebIndexedDBClientDataSet1.Filtered := False;
  Log('WIDBCDS is ' + IfThen(not WebIndexedDBClientDataSet1.Filtered, 'UN') + 'filtered');
  WebIndexedDBClientDataSet1.DisableControls;
  try
    if WebIndexedDBClientDataSet1.Active then
    begin
      Log('WIDBCDS is ' + IfThen(not WebIndexedDBClientDataSet1.IsEmpty, 'not ') + 'empty');
      Log('WIDBCDS.RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
      // Empty the DB
      if not WebIndexedDBClientDataSet1.IsEmpty then
      begin
        WebIndexedDBClientDataSet1.EmptyDataSet;
        Log('After empty WIDBCDS.RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
/////////////////  Omit this section for effect of skipping WIDBCDS.Close  ////////////////////////
///                Omitting it has negligible effect in FireFox Developer Edition
///                but increases elapsed time dramatically in Edge & Chrome
{$Define DoClose}
{$IfDef DoClose}
        WebIndexedDbClientDataset1.Close;
        Log('WIDBCDS is ' + IfThen(WebIndexedDbClientDataset1.Active,'NOT ') + 'closed');
        Log('calling await WIDBCDS.OpenAsync');
        TAwait.ExecP<Boolean>(WebIndexedDbClientDataset1.OpenAsync);
        Log('WIDBCDS is ' + IfThen(not WebIndexedDbClientDataset1.Active,'NOT ') + 'open');
        Log('WIDBCDS.RecordCount: '+ WebIndexedDBClientDataSet1.RecordCount.ToString);
{$EndIf}
      end;
      for j := 1 to StrToInt(WebEdit1.Text) do
      begin
        WebIndexedDBClientDataSet1.Append;
        FormRecord(j);
        WebIndexedDBClientDataSet1.Post;
      end;
    end;
    Log('Finished reloading WIDBCDS, RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
  finally
    WebIndexedDBClientDataSet1.Close;                 // Seems necessary to flush all buffered changes
    Log('WIDBCDS is ' + IfThen(WebIndexedDbClientDataset1.Active,'NOT ') + 'closed');
    Log('calling await WIDBCDS.EnableControls');
    await(WebIndexedDbClientDataset1.EnableControls);
    Log('WIDBCDS Controls are ' + IfThen(WebIndexedDbClientDataset1.ControlsDisabled,'NOT ') + 'Enabled');
    Log('calling await WIDBCDS.OpenAsync');
    TAwait.ExecP<Boolean>(WebIndexedDbClientDataset1.OpenAsync);
    Log('finished await WIDBCDS.OpenAsync');
    Log('WIDBCDS is ' + IfThen(not WebIndexedDbClientDataset1.Active,'NOT ') + 'Open');
    Log('WIDBCDS Controls are ' + IfThen(WebIndexedDbClientDataset1.ControlsDisabled,'NOT ') + 'Enabled');
    Log('WIDBCDS.RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
    WebDBGrid1.Refresh;
    btnClrAndRefillTable.Caption := 'Time: ' + SecondsBetween(Now, Start).ToString + ' sec';
    {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
    Log('Finished LoadWIDBCDS');
  end;
end;

procedure TForm1.btnCountRecordsClick(Sender: TObject);
begin
  if not WebIndexedDbClientDataset1.Active then
  begin
    Log('CountRecordsClick Calling WIDBCDS.OpenAsync before Count since WIDBCDS is NOT Active');
    TAwait.ExecP<Boolean>(WebIndexedDbClientDataset1.OpenAsync);
    WebLabel1.Caption := 'WIDBCDS reported Record Count:  ' + WebIndexedDbClientDataset1.RecordCount.ToString;
  end;
  Log('CountRecordsClick Calling CountWIDBCDSRecords');
  CountWIDBCDSRecords;
end;

procedure TForm1.btnUpdateTableClick(Sender: TObject);
var
  j: Integer;
  Start: TTime;
begin
  Log('Starting UpdateWIDBCDS, DB is ' + IfThen(not WebIndexedDBClientDataSet1.Active, 'not ') + 'Active');
  Start := Now;
  btnUpdateTable.Caption := 'Working...';
  {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
  if WebIndexedDBClientDataSet1.Filtered then WebIndexedDBClientDataSet1.Filtered := False;
  Log('WIDBCDS is ' + IfThen(not WebIndexedDBClientDataSet1.Filtered, 'UN') + 'filtered');
  WebIndexedDBClientDataSet1.DisableControls;
  try
    if WebIndexedDBClientDataSet1.Active then
    begin
      Log('WIDBCDS is ' + IfThen(not WebIndexedDBClientDataSet1.IsEmpty, 'not ') + 'empty');
      Log('WIDBCDS.RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
      WebIndexedDbClientDataset1.First;
      for j := 1 to StrToInt(WebEdit1.Text) do
      begin
        if WebIndexedDbClientDataset1.Eof then WebIndexedDBClientDataSet1.Append
        else WebIndexedDbClientDataset1.Edit;
        FormRecord(j);
        WebIndexedDBClientDataSet1.Post;
      end;
    end;
    Log('Finished updating WIDBCDS, RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
    for j := WebIndexedDbClientDataset1.RecordCount downto StrToInt(WebEdit1.Text) + 1 do
    begin
      WebIndexedDbClientDataset1.Last;
      WebIndexedDbClientDataset1.Delete;
    end;
  finally
    WebIndexedDBClientDataSet1.Close;                 // Seems necessary to flush all buffered changes
    Log('WIDBCDS is ' + IfThen(WebIndexedDbClientDataset1.Active,'NOT ') + 'closed');
    Log('calling await WIDBCDS.EnableControls');
    await(WebIndexedDbClientDataset1.EnableControls);
    Log('WIDBCDS Controls are ' + IfThen(WebIndexedDbClientDataset1.ControlsDisabled,'NOT ') + 'Enabled');
    Log('calling await WIDBCDS.OpenAsync');
    TAwait.ExecP<Boolean>(WebIndexedDbClientDataset1.OpenAsync);
    Log('finished await WIDBCDS.OpenAsync');
    Log('WIDBCDS is ' + IfThen(not WebIndexedDbClientDataset1.Active,'NOT ') + 'Open');
    Log('WIDBCDS Controls are ' + IfThen(WebIndexedDbClientDataset1.ControlsDisabled,'NOT ') + 'Enabled');
    Log('WIDBCDS.RecordCount: ' + WebIndexedDBClientDataSet1.RecordCount.ToString);
    WebDBGrid1.Refresh;
    btnUpdateTable.Caption := 'Time: ' + SecondsBetween(Now, Start).ToString + ' sec';
    {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
    Log('Finished UpdateWIDBCDS');
  end;
end;

procedure TForm1.SetFilter(ClassName: string);
begin
  Log('Filter button clicked for color ' + ClassName);
  WebIndexedDbClientDataset1.Filter := 'Class like ' + QuotedStr(ClassName);
  WebIndexedDbClientDataset1.Filtered := True;
//  WebDBGrid1.Refresh;
end;

procedure TForm1.btnGreenClick(Sender: TObject);
begin
  SetFilter('green');
end;

procedure TForm1.btnLocateRecordClick(Sender: TObject);
begin
  try
    Log('Locate ID: '+WebSpinEdit1.Value.ToString+' was '
      + BoolToStr(WebIndexedDbClientDataset1.Locate('id',WebSpinEdit1.Value.ToString, [loCaseInsensitive]),True));
  except
    on E:Exception do Log('Oops!! Locate raised an exception: '+ E.message) ;
  end;
end;

procedure TForm1.btnMenuTestClick(Sender: TObject);
begin
  Log('Options toggle button clicked with item '+IfThen(Options.Visible, 'visible', 'invisible'));
  WebMainMenu1.BeginUpdate;
  Options.Visible := not Options.Visible;
  WebMainMenu1.EndUpdate;
  Log('After button click, item set to '+IfThen(Options.Visible, 'visible', 'invisible'));
end;

procedure TForm1.btnRoseClick(Sender: TObject);
begin
  SetFilter('rose');
end;

procedure TForm1.btnClrFilterClick(Sender: TObject);
begin
  WebIndexedDbClientDataset1.Filtered := False;
  WebDBGrid1.Refresh;
end;

procedure TForm1.btnGrayClick(Sender: TObject);
begin
  SetFilter('gray');
end;

procedure TForm1.btnGoldenRodClick(Sender: TObject);
begin
  SetFilter('goldenRod');
end;

procedure TForm1.WebDBGrid1GetCellClass(Sender: TObject; ACol, ARow: Integer;
  AField: TField; AValue: string; var AClassName: string);
begin
  if ARow>0 then
    AClassName := WebDBGrid1.Cells[2,ARow];
end;

procedure TForm1.WebFormCreate(Sender: TObject);
begin
// Define sleep function used to allow screen updates
  asm
    window.sleep = async function(msecs) {return new Promise((resolve) => setTimeout(resolve, msecs)); }
  end;
  await(SetupWIDBCDS);
  Log('FormCreate, ' + WebIndexedDbClientDataset1.Name
  + ' record count: ' + WebIndexedDbClientDataset1.RecordCount.ToString);
//  ShowMessage('WIDBCDS record count: ' + WebIndexedDbClientDataset1.RecordCount.ToString);
  WebDBGrid1.Refresh;
  if not WebIndexedDbClientDataset1.IsEmpty then WebEdit1.Text := WebIndexedDbClientDataset1.RecordCount.ToString;
  {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
end;

procedure TForm1.SetupWIDBCDS;
var
  i: Integer;
const
  DBFIELDS: array[0..2] of string = ('PSIP', 'StartTime', 'Class');
begin
  Log('Setting up to (re)open WIDBCDS');
  WebIndexedDbClientDataset1.FieldDefs.Clear;
  // add key field
  WebIndexedDbClientDataset1.FieldDefs.Add('id', ftInteger, 0, False);
  // add normal fields
  for i := 0 to Length(DBFIELDS) - 1 do
  begin
    if (DBFIELDS[i] = 'StartTime') then
      WebIndexedDbClientDataset1.FieldDefs.Add(DBFIELDS[i], ftDateTime)
    else
      WebIndexedDbClientDataset1.FieldDefs.Add(DBFIELDS[i], ftString);
  end;
  TAwait.ExecP<Boolean>(WebIndexedDbClientDataset1.OpenAsync);
  btnUpdateTableClick(Self);
  Log('WIDBCDS is ' + IfThen(not WebIndexedDbClientDataset1.Active, 'not ')
    + 'Active and ' + IfThen(not WebIndexedDbClientDataset1.IsEmpty, 'not ') + 'Empty');
//  if WebIndexedDbClientDataset1.IsEmpty then btnUpdateTableClick(Self);
end;

procedure TForm1.CountWIDBCDSRecords;
var i: Integer;
begin
  WebLabel1.Caption := 'WIDBCDS Record Count:  (working)';
  {$IFDEF PAS2JS} asm await sleep(10) end; {$ENDIF}
  Log('Count WIDBCDS Records started');
  Log('WIDBCDS is ' + IfThen(not WebIndexedDbClientDataset1.Active,'NOT ') + 'active');
  i := 0;
  if WebIndexedDbClientDataset1.Active then
  begin
    WebIndexedDbClientDataset1.DisableControls;
    Log('WIDBCDS Controls are '+IfThen(WebIndexedDbClientDataset1.ControlsDisabled, 'NOT ')+'enabled');
    WebIndexedDbClientDataset1.First;
    while not WebIndexedDbClientDataset1.Eof do
    begin
      Inc(i);
      WebIndexedDbClientDataset1.Next;
    end;
    while WebIndexedDbClientDataset1.ControlsDisabled do
    begin
      WebIndexedDbClientDataset1.EnableControls;
      {$IFDEF PAS2JS} asm await sleep(100) end; {$ENDIF}
    end;
  end;
  Log('Count WIDBCDS Records found: ' + i.ToString);
  Log('WIDBCDS Controls are '+IfThen(WebIndexedDbClientDataset1.ControlsDisabled, 'NOT ')+'enabled');
  WebLabel1.Caption := 'WIDBCDS Records Counted:  ' + i.ToString;
end;

end.
